@@ -55,7 +55,7 @@ CVBooster <- R6::R6Class(
 #'
 #' @examples
 #' # See https://github.com/fabsig/GPBoost/tree/master/R-package for more examples
-#' 
+#' \donttest{
 #' library(gpboost)
 #' data(GPBoost_data, package = "gpboost")
 #' 
@@ -77,6 +77,7 @@ CVBooster <- R6::R6Class(
 #'                 use_gp_model_for_validation = TRUE)
 #' print(paste0("Optimal number of iterations: ", cvbst$best_iter,
 #'              ", best test error: ", cvbst$best_score))
+#' }
 #' @importFrom data.table data.table setorderv
 #' @author Authors of the LightGBM R package, Fabio Sigrist
 #' @export
@@ -480,6 +481,7 @@ gpb.cv <- function(params = list()
         vecchia_ordering <- gp_model$.__enclos_env__$private$vecchia_ordering
         vecchia_pred_type <- gp_model$.__enclos_env__$private$vecchia_pred_type
         num_neighbors_pred <- gp_model$.__enclos_env__$private$num_neighbors_pred
+        cg_delta_conv_pred <- gp_model$.__enclos_env__$private$cg_delta_conv_pred
         cov_function <- gp_model$get_cov_function()
         cov_fct_shape <- gp_model$get_cov_fct_shape()
         cov_fct_taper_range <- gp_model$get_cov_fct_taper_range()
@@ -498,8 +500,6 @@ gpb.cv <- function(params = list()
                                           vecchia_approx = vecchia_approx,
                                           num_neighbors = num_neighbors,
                                           vecchia_ordering = vecchia_ordering,
-                                          vecchia_pred_type = vecchia_pred_type,
-                                          num_neighbors_pred = num_neighbors_pred,
                                           cluster_ids = cluster_ids,
                                           likelihood = gp_model$get_likelihood_name(),
                                           free_raw_data = TRUE)
@@ -509,7 +509,10 @@ gpb.cv <- function(params = list()
                                              group_rand_coef_data_pred = group_rand_coef_data_pred,
                                              gp_coords_pred = gp_coords_pred,
                                              gp_rand_coef_data_pred = gp_rand_coef_data_pred,
-                                             cluster_ids_pred = cluster_ids_pred)
+                                             cluster_ids_pred = cluster_ids_pred,
+                                             vecchia_pred_type = vecchia_pred_type,
+                                             num_neighbors_pred = num_neighbors_pred,
+                                             cg_delta_conv_pred = cg_delta_conv_pred)
           if (has_custom_eval_functions) {
             # Note: Validation using the GP model is only done in R if there are custom evaluation functions in eval_functions, 
             #        otherwise it is directly done in C++. See the function Eval() in regression_metric.hpp
@@ -632,7 +635,7 @@ gpb.cv <- function(params = list()
                                                              , reshape = FALSE )
     }
     
-    message("Fitting GPModel on out-of-sample data...")
+    # message("Fitting GPModel on out-of-sample data...") # message removed in version 0.7.8
     if(gp_model$get_likelihood_name() == "gaussian"){
       gp_model$fit(y = data$.__enclos_env__$private$info$label - pred_fixed_effect_OOS)
     }
@@ -915,7 +918,6 @@ get.param.combination <- function(param_comb_number, param_grid) {
 #'
 #' @examples
 #' # See https://github.com/fabsig/GPBoost/tree/master/R-package for more examples
-#' 
 #' \donttest{
 #' library(gpboost)
 #' data(GPBoost_data, package = "gpboost")
